/* $Id: TuningNonZero.java,v 1.8 2014/03/05 07:03:57 mit Exp $
/* TuningNonZero v0.3.4 Type3 */

import java.text.DecimalFormat;

public class TuningNonZero {
    private final int KEY_BOARD = 88;
    private final int OCTAVE = 12;
    private final int P4TH = 5;
    private final int P5TH = 7;
    private final int CENT = 1200;
    private final double REVOCT = 1.0/OCTAVE;

    private boolean isCalc;
    private int FROM_KEY = 33-1;  // f
    private int TO_KEY = 45-1;    // f
    private int WOUND_WIRE = 28;
    private double BEAT = 1.0;
    private double[] Cent;
    private double[] Inha;

    private DecimalFormat df1;
    private Interval IN;
    private MinMax MM;

    public TuningNonZero() {
	df1 = new DecimalFormat("#0.0##");
	IN = new Interval();

	resetCent();
    }
    public void Tuning(double[] inha, double[] cent) {
	Cent = cent;
	Inha = inha;

	tuneBass();
	tuneTreble();

	isCalc = true;
    }
    public double[] getCent() {
	return Cent;
    }
    public double getCent(int num) {
	return Cent[num];
    }
    public double getICent(int num) {
	return Inha[num]+Cent[num];
    }
    public void resetCent() {
	Cent = new double[KEY_BOARD];

	for (int i = 0; i < KEY_BOARD; i++)
	    Cent[i] = 0;
    }
    public void setCent(int num, double ce) {
	Cent[num] = ce;
    }
    public void setBeat(double beat) {
	BEAT = beat;

	this.resetIsCalc();
    }
    public double getBeat() {
	return BEAT;
    }
    public void resetIsCalc() {
	isCalc = false;
    }
    public boolean isCalc() {
	return isCalc;
    }
    //
    public void tuneBeat(int key0, int key1, double beat) {
	double be = calcBeat(key0, key1);
	double sa = Math.abs(beat-be);

	while (sa > 0.005) {
	    sa = Math.abs(beat-be);
	    be = calc_beat(key0, key1, beat);
	}
    }
    public double calc_beat(int k0, int k1, double beat) {
	double dc = 0.01;
	double b0 = calcBeat(k0, k1);
	double sa = Math.abs(b0-beat);
	if (sa > 0.3)
	    dc = 0.5; //
	else if (sa > 0.1)
	    dc = 0.05;
	else
	    dc = 0.01;

	Cent[k1] = (b0 < beat) ? Cent[k1]+dc : Cent[k1]-dc;

	return b0;
    }
    public double calcBeat(int key0, int key1) {
	int mul = 5;
	double[] f0 = calcPartial(mul, key0);
	double[] f1 = calcPartial(mul, key1);
	double[] be = new double[mul];

	double min = Double.MAX_VALUE;
	for (int i = 0; i < mul; i++) {
	    for (int j = 0; j < mul; j++) {
		be[j] = f1[j]-f0[i];
		if (Math.abs(min) > Math.abs(be[j]))
		    min = be[j];
	    }
	}

	return min;
    }
    public double[] calcPartial(int mul, int key) {
	double[] f = new double[mul];

	for (int i = 0; i < mul; i++)
	    f[i] = calcFreq(i+1, key);

	return f;
    }
    public double calcFreq(int par, int key) {
	return ctof(getFreq(key)*par, calc_inha(key, par)+Cent[key]);
    }
    public double ctof(double f, double c) {
        return f*Math.pow(2, c/CENT);
    }
    public double getFreq(int key) {
	return 440*Math.pow(2, (key-48)*REVOCT); // 49-1
    }
    //
    public void tuneBass() {
	int[] ratio = IN.getIratio(OCTAVE);

	for (int i = FROM_KEY-1; i >= 0; i--) {
	    tune_bass(i, ratio);
	}
    }
    public void tune_bass(int key, int[] ratio) {
        int keyh = key+ratio[2];
        int len = 400;
        double be;
        double ch = (key < 13) ? Cent[keyh]-500 : Cent[keyh]-200;
        double dc = ch/len;
	MM = new MinMax(len);

        Cent[key] = Cent[keyh];
        for (int i = 0; i < len; i++) {
            be = partial_bass(keyh, key, ratio);
	    MM.minmax(be, Cent[key], i);
	    Cent[key] += dc;
        }
	Cent[key] = MM.getCent();
    }
    public double partial_bass(int keyh, int keyl, int[] ratio) {
	int ww = WOUND_WIRE-1; // -1Ǵ̵

	if (ww >= 0 && keyl < WOUND_WIRE) {
	    int len = 5;
	    double df = 0;
	    double[] be = new double[len];

            for (int i = 0; i < len; i++)
                be[i] = get_mbeats(keyh, keyl, ratio, i+1);
            for (int j = 0; j < len-1; j++)
                df += Math.abs(Math.abs(be[j+1])-Math.abs(be[j]));

	    return df;
	} else {
	    int[] rat4th = IN.getIratio(P4TH);
	    keyh = keyl+rat4th[2];
	    double b4 = get_ibeats(keyh, keyl, rat4th);

	    return Math.abs(b4-BEAT);
	}
    }
    public double get_mbeats(int keyh, int keyl, int[] ratio, int mul) {
	return get_ibeats(keyh, keyl, calc_multi(ratio, mul));
    }
    public double getMbeats(int key, int inte, int mul) {
	int[] ratio = IN.getIdata(inte); // -б
	int keyh = key+ratio[2];

	return get_mbeats(keyh, key, ratio, mul);
    }
    public int[] calc_multi(int[] ratio, int mul) {
	int[] rat = new int[3];

	if (mul < 1) {
	    System.out.println("*** Multi input error. ***");
	    mul = 1;
	}
	rat[0] = ratio[0]*mul; // low
	rat[1] = ratio[1]*mul; // hi
	rat[2] = ratio[2];

	return rat;
    }
    public double get_ibeats(int keyh, int keyl, int[] ratio) {
	double fh = calcFreq(ratio[1], keyh);
	double fl = calcFreq(ratio[0], keyl);

	return (fh-fl);
    }
    public double calc_inha(int key, int part) {
	return Inha[key]*part*part;
    }
    //
    public void tuneTreble() {
	int[] ratio = IN.getIratio(OCTAVE);

	for (int i = TO_KEY+1; i < KEY_BOARD; i++) { // +1
	    tune_treble(i, ratio);
	}
    }
    public void tune_treble(int key, int[] ratio) {
        int keyl = key-ratio[2];
        double dc = 0.01;
        Cent[key] = Cent[keyl];
        double be0 = 0;
        double be = partial_high(key, keyl, ratio);

        try {
            do {
                Cent[key] += dc;
                be0 = be;
                be = partial_high(key, keyl, ratio);
            } while (be < be0);
        } catch (Exception e) {
            System.out.println(e);
        }
    }
    public double partial_high(int keyh, int keyl, int[]ratio) {
	if (keyh > TO_KEY+OCTAVE)
	    return get3KeyBeat(keyh);
	else {
	    int[] rat4th = IN.getIratio(P4TH);
	    keyl = keyh-rat4th[2];
	    double b4 = get_ibeats(keyh, keyl, rat4th);

	    return Math.abs(b4-BEAT);
	}
    }
    public double get3KeyBeat(int key) {
	double f0 = calcFreq(1, key);
	double be0 = f0-calcFreq(2, key-OCTAVE);   // octave
	double be1 = f0-calcFreq(4, key-OCTAVE*2); //2octave

	return Math.abs(be0+be1);
    }
    //
    public String toString() {
        StringBuffer sb = new StringBuffer();

	sb.append("FromKey:"+FROM_KEY);
        sb.append(" ToKey:"+TO_KEY);
        sb.append(" WoundWire:"+WOUND_WIRE);

	return sb.toString();
    }
    public String dform(double n) {
        return df1.format((float)n);
    }
}