/* $Id: laying.java,v 1.39 2014/03/07 07:18:18 mit Exp $
/* laying.java v0.2 */

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.*;
import java.text.DecimalFormat;

public class laying extends Applet 
    implements ActionListener, AdjustmentListener, ItemListener,
	       MouseListener, MouseMotionListener
{
    private static final long serialVersionUID = 1L;
    private final int KB = 88;
    private final int OCT = 12;
    private final int INHA = 0;
    private final int CENT = 1;
    private final int LAYI = 2;
    private final int BEAT = 3;
    private final int TRAC = 4;
    private final int[] INTER = {3, 4, 5, 7, 12};
    private final String[] Graph =
    {"Inhar.", "Cent", "Laying", "Beat", "Trace"};
    private final String[] OCTAVE_NAME = 
    {"A2","A1","A","a","a1","a2","a3","a4"};
    private final String[] KEY_NAME =
    {"A","B","H","C","Cis","D","Es","E","F","Fis","G","Gis"};

    private boolean TUNEsw;
    private boolean TEXTsw;
    private int FNT;
    private int W, H, H2, W2;
    private int WOUND = 28-1;
    private int GRAP = CENT;
    private double BeatA = 1.1;
    private double BeatB = 0.38;
    private double Beat = 1.0;
    private double[] Inha;

    private Button[] btGraph;
    private Button btTune;
    private Button btP;
    private Button btM;
    private Button btText;
    private Button btWide;
    private Button btNarrow;
    private Button btBeatP;
    private Button btBeatM;
    private Scrollbar scrBeat1;
    private Scrollbar scrBeat2;
    private Scrollbar scrBeat3; // octave
    private Scrollbar scrColor; // beat-color
    private Scrollbar scrTreble;
    private Scrollbar scrBass;
    private Scrollbar scrPitch;
    private Scrollbar scrWire;
    private Choice chLaying;
    private Label lbBeat1;
    private Label lbBeat2;
    private Label lbBeat3;
    private Label lTreble;
    private Label lBass;
    private Label lPitch;
    private Label lWire;
    private Label lColor;
    private TextField tfData;
    private TextArea taA;
    private CardLayout Card;
    private Panel Deck;

    private Graphics gi;
    private Image ig;
    private Font Fnt;
    private FontMetrics fm;
    private Canvas can;
    private Panel PN, PE, PW, PS;
    private DecimalFormat df1 = new DecimalFormat("#0.0#");

    private MakeInharmonicity MI;
    private AssortLayings AL;
    private Interval IN;

    public void init() {
	String fnt = getParameter("font");
        FNT = (fnt != null) ? Integer.parseInt(fnt) : 12;
        Fnt = new Font("SansSerif", Font.PLAIN, FNT);
        setFont(Fnt);
	fm = getFontMetrics(Fnt);

	AL = new AssortLayings();
	int num = AL.get_laying_size();
	buildLayout(num);
	set_enabled();
	init_seting();
    }
    public void buildLayout(int num) {
        setBackground(Color.lightGray);
        setLayout(new BorderLayout());

	int len = Graph.length;
        PW = new Panel();
        PW.setLayout(new GridLayout(len+2,1,0,0));
        add(PW, "West");

	btGraph = new Button[len];
        for (int i = 0; i < len; i++) {
            btGraph[i] = new Button(Graph[i]);
            btGraph[i].addActionListener(this);
            PW.add(btGraph[i]);
        }

	PW.add(new Label(""));

        btText = new Button("Graph.");
        btText.addActionListener(this);
        PW.add(btText);
	//---

        PN = new Panel();
        PN.setLayout(new FlowLayout());
        add(PN, "North");

        tfData = new TextField(28);
        tfData.setEditable(false);
        PN.add(tfData);

	btTune = new Button("Tune ");
        btTune.addActionListener(this);
        PN.add(btTune);

	chLaying = new Choice();
	for (int i = 0; i < num; i++)
            chLaying.addItem(AL.get_layings_name(i));
        PN.add(chLaying);
        chLaying.addItemListener(this);
	//---

        PE = new Panel();
        PE.setLayout(new GridLayout(7,1,0,0));
        add(PE, "East");

	btP = new Button("+");
        btP.addActionListener(this);
        PE.add(btP);

	btM = new Button("-");
        btM.addActionListener(this);
        PE.add(btM);

	PE.add(new Label(""));

	btWide = new Button("==");
        btWide.addActionListener(this);
        PE.add(btWide);

	btNarrow = new Button("=");
        btNarrow.addActionListener(this);
        PE.add(btNarrow);

	btBeatP = new Button("x2");
        btBeatP.addActionListener(this);
        PE.add(btBeatP);

	btBeatM = new Button("/2");
        btBeatM.addActionListener(this);
        PE.add(btBeatM);
	//---

	PS = new Panel();
	PS.setLayout(new GridLayout(1,2,0,0));
	add(PS, "South");

	Panel p0 = new Panel();
	p0.setLayout(new GridLayout(4,2,0,0));
	PS.add(p0);

	Panel p1 = new Panel();
	p1.setLayout(new GridLayout(4,2,0,0));
	PS.add(p1);

	scrTreble = new Scrollbar(Scrollbar.HORIZONTAL,87,10,0,130);
        p0.add(scrTreble);
        scrTreble.setBlockIncrement(10);
        scrTreble.addAdjustmentListener(this);

	lTreble = new Label("TrebleG:___", Label.LEFT);
        p0.add(lTreble);

	scrBass = new Scrollbar(Scrollbar.HORIZONTAL,60,10,0,130);
        p0.add(scrBass);
        scrBass.setBlockIncrement(10);
        scrBass.addAdjustmentListener(this);

        lBass = new Label("BassG:___", Label.LEFT);
        p0.add(lBass);

        scrPitch = new Scrollbar(Scrollbar.HORIZONTAL,55,10,1,100);
        p0.add(scrPitch);
        scrPitch.setBlockIncrement(10);
        scrPitch.addAdjustmentListener(this);

        lPitch = new Label("A49I:___", Label.LEFT);
        p0.add(lPitch);

        scrWire = new Scrollbar(Scrollbar.HORIZONTAL,28,1,0,49); //49-1
        p0.add(scrWire);
        scrWire.setBlockIncrement(10);
        scrWire.addAdjustmentListener(this);

        lWire = new Label("WoundW:___", Label.LEFT);
        p0.add(lWire);

	lbBeat1 = new Label("Beat1:___", Label.RIGHT);
        p1.add(lbBeat1);

	scrBeat1 = new Scrollbar(Scrollbar.HORIZONTAL,74,1,0,101);
        p1.add(scrBeat1);
        scrBeat1.setBlockIncrement(10);
        scrBeat1.addAdjustmentListener(this);

	lbBeat2 = new Label("Beat2:___", Label.RIGHT);
        p1.add(lbBeat2);

	scrBeat2 = new Scrollbar(Scrollbar.HORIZONTAL,55,1,0,101);
        p1.add(scrBeat2);
        scrBeat2.setBlockIncrement(10);
        scrBeat2.addAdjustmentListener(this);

	lbBeat3 = new Label("Beat3:___", Label.RIGHT); // octave
        p1.add(lbBeat3);

	scrBeat3 = new Scrollbar(Scrollbar.HORIZONTAL,10,1,0,51);
        p1.add(scrBeat3);
        scrBeat3.setBlockIncrement(10);
        scrBeat3.addAdjustmentListener(this);

	lColor = new Label("BeatColor:0.9", Label.RIGHT);
	p1.add(lColor);

	scrColor = new Scrollbar(Scrollbar.HORIZONTAL,90,1,0,100);
	p1.add(scrColor);
	scrColor.addAdjustmentListener(this);
	//---

        can = new myCanvas();

        can.addMouseListener(this);
        can.addMouseMotionListener(this);

        Deck = new Panel();
        Card = new CardLayout();
        Deck.setLayout(Card);

        Panel card1 = new Panel();
        card1.add(can);
        Deck.add(card1, "card1");

        taA = new TextArea("", 16, 40, TextArea.SCROLLBARS_BOTH);
        taA.setFont(Fnt);
        taA.setEditable(false);

        ScrollPane card2 = new ScrollPane();
        card2.add(taA);
        Deck.add(card2, "card2");

        add(Deck, "Center");
    }
    public void start() {
	W = getCanvasD().width;
        H = getCanvasD().height;

	can.setSize(W, H);
        ig = createImage(W, H);
        gi = ig.getGraphics();
        gi.setFont(Fnt);

	H2 = H/2;
	W2 = W/2;
	BYH = H/10;
	DK = W/(double)KB;
	DL = W/(double)(OCT+1);
	DL2 = DL/2.0;
	DOT = (DK < 2) ? 2 : (int)DK;
	DSF = DOT/2;
	DOT2 = (int)(DL/4.0);
	DSF2 = DOT2/2;

	KEY = 37-1;
	write_data();
    }
    public Dimension getCanvasD() {
        Dimension de = PE.getSize();
        Dimension dn = PN.getSize();
        Dimension ds = PS.getSize();
        Dimension dw = PW.getSize();
        Dimension da = getSize();
        int cw = da.width-(de.width+dw.width);
        int ch = da.height-(dn.height+ds.height)-6;
        Dimension can = new Dimension(cw, ch);

        return can;
    }
    public void set_enabled() {
	boolean lsw = (GRAP == LAYI);
	btWide.setEnabled(lsw);
	btNarrow.setEnabled(lsw);

	boolean pmsw = (GRAP == BEAT || GRAP == CENT);
	btP.setEnabled(pmsw);
	btM.setEnabled(pmsw);

	boolean cosw = (GRAP == BEAT || GRAP == LAYI || GRAP == TRAC);
	lColor.setEnabled(cosw && !TEXTsw);
    }
    public void init_seting() {
        MI = new MakeInharmonicity();
	AL.setLayings(MI.getInha());
	IN = new Interval();

	CMAX = 100;
	BMAX = 20;

	init_beat1();
	init_beat2();
	init_beat3();

	set_treble_grade(false);
	set_bass_grade(false);
	set_pitch_inha(false);
	set_wound_wire(false);
    }
    public void reset_tune() {
	TUNEsw = !TUNEsw;

	reset_tune_label();
    }
    public void clear_tune() {
	TUNEsw = false;

	reset_tune_label();
    }
    public void reset_tune_label() {
	btTune.setLabel(TUNEsw ? "Laying" : "Tune ");

	if (TUNEsw)
	    AL.tuneExtension();
	else
	    AL.resetCent();
    }
    //
    public void mouseMoved(MouseEvent me) {
	if (Msw) {
	    if (GRAP == TRAC)
		return;
	    else if (GRAP == LAYI)
		check_laying_key(me.getX());
	    else
		check_key(me.getX());

	    disp_sw();
	}
    }
    public void check_laying_key(int mx) {
	int lay = AL.get_from_key();
	KEY = (int)(mx/DL)+lay;
	if (KEY < lay)
	    KEY = lay;
	else if (KEY > lay+OCT)
	    KEY = lay+OCT;
    }
    public void check_key(int mx) {
        KEY = (int)(mx/DK);
        if (KEY < 0)
            KEY = 0;
        else if (KEY >= KB)
            KEY = KB-1;
    }
    public void mouseDragged(MouseEvent me) {}
    public void mousePressed(MouseEvent m) {
	Msw = !Msw;
    }
    public void mouseClicked(MouseEvent m) {}
    public void mouseReleased(MouseEvent m){}
    public void mouseEntered(MouseEvent m) {}
    public void mouseExited (MouseEvent m) {}
    //
    public void actionPerformed(ActionEvent e) {
	for (int i = 0; i < Graph.length; i++) {
	    if (e.getSource() == btGraph[i]) {
		GRAP = i;
		break;
	    }
	}

	if (e.getSource() == btTune)
	    reset_tune();

	else if (e.getSource() == btP)
	    set_pm(true);
	else if (e.getSource() == btM)
	    set_pm(false);

	else if (e.getSource() == btBeatP)
	    set_bye(true);
	else if (e.getSource() == btBeatM)
	    set_bye(false);

	else if (e.getSource() == btNarrow)
	    set_wide(false);
	else if (e.getSource() == btWide)
	    set_wide(true);

	else if (e.getSource() == btText)
	    disp_text();

	set_enabled();
	disp_sw();
    }
    public void disp_sw() {
        if (TEXTsw)
            re_write();
        else
            can.repaint();

        write_data();
    }
    public void disp_text() {
       TEXTsw = !TEXTsw;

        btText.setLabel(TEXTsw ? "Text " : "Graph.");
        Card.next(Deck);
    }
    public void set_pm(boolean sw) {
	switch(GRAP) {
	case LAYI:
	    break;
	case CENT:
	    if (sw)
		CMAX /= 2;
	    else
		CMAX *= 2;

	    if (CMAX < 1)
		CMAX = 1;
	    break;
	case BEAT:
	    if (sw)
		BMAX /= 2;
	    else
		BMAX *= 2;

	    if (BMAX < 1)
		BMAX = 1;
	    break;
	default:
	    break;
	}
    }
    public void set_bye(boolean sw) {
	if (sw)
	    Bby /= 2;
	else
	    Bby *= 2;

	set_beat1();
	set_beat2();
    }
    public void set_wide(boolean sw) {
	if (sw)
	    BAND *= 2;
	else
	    BAND /= 2;
    }
    //
    public void adjustmentValueChanged(AdjustmentEvent e) {
	if (e.getSource() == scrTreble)
            set_treble_grade(true);
        if (e.getSource() == scrBass)
            set_bass_grade(true);
        if (e.getSource() == scrPitch)
            set_pitch_inha(true);
        if (e.getSource() == scrWire)
            set_wound_wire(true);

	if (e.getSource() == scrBeat1)
            set_beat1();
	if (e.getSource() == scrBeat2)
	    set_beat2();
	if (e.getSource() == scrBeat3)
	    set_beat3();

	if (e.getSource() == scrColor)
	    set_color();

	disp_sw();
    }
    public void set_color() {
	float sft = scrColor.getValue()/100f;
	IN.initColor(sft);

	lColor.setText("BeatColor:"+sft);
    }
    public void set_treble_grade(boolean sw) {
	double it = scrTreble.getValue()/1000.0;
        lTreble.setText("TrebleG:"+(float)it);
        MI.setTrebleGrade(it);
        Inha = MI.getInha();
	if (sw)
	    set_inha();
    }
    public void set_bass_grade(boolean sw) {
	double ib = scrBass.getValue()/1000.0;
        lBass.setText("BassG:"+(float)ib);
        MI.setBassGrade(ib);
        Inha = MI.getInha();
	if (sw)
	    set_inha();
    }
    public void set_pitch_inha(boolean sw) {
	double a49 = scrPitch.getValue()/100.0;
        lPitch.setText("A49I:"+(float)a49);
        MI.setA49i(a49);
        Inha = MI.getInha();
	if (sw)
	    set_inha();
    }
    public void set_wound_wire(boolean sw) {
	int ww = scrWire.getValue();
        lWire.setText("WoundW:"+ww);
        MI.setWoundWire(ww);
        Inha = MI.getInha();
	WOUND = ww;
	if (sw)
	    set_inha();
    }
    public void set_inha() {
	AL.setLayings(Inha); // cent-0

	set_beat3();
    }
    public void init_beat() {
	AL.is_calc();
    }
    //
    public void set_beat3() {
	Beat = scrBeat3.getValue()/10.0;

	init_beat3();
    }
    public void init_beat3() {
	AL.setBeat3(Beat);
	set_beat3_label(Beat);
    }
    public void set_beat3_label(double be) {
	lbBeat3.setText("Beat(octave):"+(float)be);
    }
    //
    public void set_beat2() {
	BeatB = (scrBeat2.getValue()-50)/Bby;

	init_beat2();
    }
    public void init_beat2() {
	AL.setBeat2(BeatB);
	AL.is_calc();
	set_beat2_label(BeatB);
    }
    public void set_beat2_label(double be) {
	int[] ra = AL.select_ratio();
        lbBeat2.setText("Beat("+IN.getIinter(ra[1])+"):"+dform(be));
    }
    //
    public void set_beat1() {
	BeatA = (scrBeat1.getValue()-50)/Bby;

	init_beat1();
    }
    public void init_beat1() {
	AL.setBeat1(BeatA);
	AL.is_calc();
	set_beat1_label(BeatA);
    }
    public void set_beat1_label(double be) {
	int[] ra = AL.select_ratio();
        lbBeat1.setText("Beat("+IN.getIinter(ra[0])+"):"+dform(be));
    }
    //
    public void itemStateChanged(ItemEvent e) {
	int num = chLaying.getSelectedIndex();
	clear_tune();
	AL.setLaying(num);
	AL.resetIsCalc();

	set_beat1();
	set_beat2();

        disp_sw();
    }
    //
    public String dform(double n) {
        return df1.format((float)n);
    }
    public String getKeyName(int key) {
        return (key+1)+"("+getKey(key)+")";
    }
    public String getKey(int k) {
	return KEY_NAME[k%12];
    }
    public void write_data() {
	StringBuffer sb = new StringBuffer(24);

	switch (GRAP) {
	case LAYI:
	case CENT:
	    sb.append(getKeyName(KEY));
	    sb.append(": "+dform(AL.calciFreq(1, KEY))+"[Hz]: ");
	    sb.append(dform(AL.getCent(KEY)));
	    //sb.append(dform(AL.getICent(KEY, 1)));
	    sb.append(" (c_diff: ");
	    sb.append(dform(AL.centDiff())+")");
	    break;
	case INHA:
	    sb.append(getKeyName(KEY));
	    sb.append(": "+dform(AL.calciFreq(1, KEY))+"[Hz]: ");
	    sb.append(dform(Inha[KEY]));
	    break;
	case BEAT:
	    sb.append(getKeyName(KEY));
	    int[] intn = new int[3];
	    for (int i = 0; i < INTER.length; i++) {
		if (KEY >= KB-INTER[i])
		    sb.append(":--- ");
		else {
		    intn = IN.getIdata(INTER[i]);
		    sb.append(": "+dform(AL.getiBeat(KEY, intn)));
		}
	    }
	    break;
	case TRAC:
	    sb.append("cent diff: ");
	    sb.append(dform(AL.centDiff()));
	    break;
	default:
	    sb.append(getKeyName(KEY));
	    break;
	}

        tfData.setText(sb.toString());
    }
    public void re_write() {
	switch (GRAP) {
	case LAYI:
	    write_laying();
	    break;
        case INHA:
	    write_inha();
	    break;
	case CENT:
	    write_cent();
	    break;
	case BEAT:
	    write_beat();
	    break;
	case TRAC:
	    write_trace();
	    break;
	default:
	    break;
	}
    }
    public void write_laying() {
	StringBuffer str = new StringBuffer(1024);
	int len = INTER.length;
	int[][] intn = new int[len][3];

	str.append("Key#: [Hz]: [cent]");
	for (int i = 0; i < len; i++) {
	    str.append(": "+IN.getIinter(INTER[i]));
	    intn[i] = IN.getIdata(INTER[i]);
	}
	str.append("\n");
        str.append("---------------------------------------\n");

	int ii, lay = AL.get_from_key();
	for (int i = 0; i <= OCT; i++) {
	    ii = i+lay;
	    str.append(getKeyName(ii)+": ");
	    str.append(dform(AL.calciFreq(1, ii))+": ");
	    str.append(dform(AL.getCent(ii)));
	    //str.append(dform(AL.getICent(ii, 1)));
	    for (int j = 0; j < len; j++) {
		str.append(": "+dform(AL.getiBeat(ii, intn[j])));
	    }
	    str.append("\n");
	}

	taA.setText(str.toString());
    }
    public void write_inha() {
        StringBuffer str = new StringBuffer(1024);
        str.append(" Key#: [Hz]: inharmonicity \n");
        str.append("------------------------- \n");
        for (int i = 0; i < KB; i++) {
            str.append(getKeyName(i)+": ");
            str.append(dform(AL.calciFreq(1, i))+": ");
	    str.append(dform(Inha[i]));
            str.append("\n");
        }

        taA.setText(str.toString());
    }
    public void write_cent() {
        StringBuffer str = new StringBuffer(1024);
        str.append("Key#: [Hz]: [cent] \n");
        str.append("------------------- \n");
        for (int i = 0; i < KB; i++) {
            str.append(getKeyName(i)+": ");
            str.append(dform(AL.calciFreq(1, i))+": ");
	    str.append(dform(AL.getCent(i)));
	    //str.append(dform(AL.getICent(i, 1)));
            str.append("\n");
        }

        taA.setText(str.toString());
    }
    public void write_beat() {
	StringBuffer str = new StringBuffer(1024);
	int len = INTER.length;
	int[][] intn = new int[len][3];

        str.append("Key#:  [Hz]");
	for (int i = 0; i < len; i++) {
	    str.append(": "+IN.getIinter(INTER[i]));
	    intn[i] = IN.getIdata(INTER[i]);
	}
	str.append("\n");
	str.append("------------------------------ \n");

	for (int j = 0; j < KB; j++) {
	    str.append(getKeyName(j)+": ");
	    str.append(dform(AL.calciFreq(1, j)));
	    for (int i = 0; i < len; i++) {
		if (j >= KB-intn[i][2])
		    str.append(":___ ");
		else
		    str.append(": "+dform(AL.getiBeat(j, intn[i])));
	    }
	    str.append("\n");
	}

	taA.setText(str.toString());
    }
    public void write_trace() {
	StringBuffer str = new StringBuffer(1024);
        str.append("Key#--Key#: Interval: Beat \n");
        str.append("-------------------------- \n");

	int[] la = AL.select_laying();
	int[] ke = AL.select_keys();
	int k1, k2;

	for (int i = 0; i < la.length; i++) {
	    k1 = ke[i];
	    k2 = ke[i]+la[i];
	    str.append(getKeyName((la[i] < 0) ? k2 : k1));
	    str.append((la[i] < 0) ? "<-" : "->");
	    str.append(getKeyName((la[i] < 0) ? k1 : k2)+": ");
	    str.append(IN.getIinter(la[i])+": ");
	    str.append(dform(AL.getiBeat(ke[i], IN.getIdata(la[i]))));
	    str.append("\n");
	}

	taA.setText(str.toString());
    }
    //
    class myCanvas extends Canvas {
        private static final long serialVersionUID = 1L;
        public void update (Graphics g) {
            can.paint(g);
        }
        public void paint(Graphics g) {
            paintI();
            g.drawImage(ig, 0, 0, this);
        }
    }

    private double DK; // key
    private double DH; // cent
    private double DL; // laying
    private double DB; // beat
    private double DL2; // "
    private double BYH; // inha.
    private double CMAX; // cent
    private double BMAX; // beat
    private double BAND = 100; // band
    private double Bby = 20;

    private int DOT, DSF;
    private int DOT2, DSF2;
    private int mX, KEY;
    private boolean Msw;

    public void paintI() {
	back_color(gi);
	init_calc();

	switch (GRAP) {
	case LAYI:
	    disp_laying(gi);
	    break;
        case INHA:
	    disp_inha(gi);
	    break;
	case CENT:
	    disp_cent(gi);
	    break;
	case BEAT:
	    disp_beat(gi);
	    break;
	case TRAC:
	    disp_trace(gi);
	    break;
	default:
	    break;
	}

	//color_test(gi);
    }
    public void back_color(Graphics g) {
        g.setColor(Color.black);
        g.fillRect(0, 0, W, H);
    }
    public void color_test(Graphics g) {
	int hei = 10, len = IN.getLen();
	double dx = W/(double)len;

	for (int i = 0; i < IN.getLen(); i++) {
	    g.setColor(IN.getColor(i));
	    g.fillRect((int)(dx*i), H-hei, (int)dx, hei);
	}
    }
    public void init_calc() {
	DH = H/CMAX;
	DB = H/BMAX;
    }
    //
    public void disp_laying(Graphics g) {
	int[] ra = AL.select_ratio();
	int[] pa1 = IN.getIdata(ra[0]);
	int[] pa2 = IN.getIdata(ra[1]);
	Color co1 = IN.getColor(ra[0]);
	Color co2 = IN.getColor(ra[1]);
	int sttk = 33-1;
	int endk = 45-Math.min(pa1[2], pa2[2]);

	int kmax = endk-sttk;
	double dy = H/(double)kmax;
	int dsf = (int)(dy/2);
	int bnd = (int)(dy/4);
	double val1, val2;
	int dh1, dh2, key;

	disp_lay_line(g, kmax, dy);

	for (int i = 0; i < kmax; i++) {
	    key = i+sttk;
	    val1 = AL.getiBeat(key, pa1);
	    val2 = AL.getiBeat(key, pa2);

	    dh1 = (int)(i*dy);
	    dh2 = (int)(i*dy+dsf);

	    draw_bar(g, key, pa1, dh1, val1, bnd, co1);
	    draw_bar(g, key, pa2, dh2, val2, bnd, co2);
	}
    }
    public void draw_bar(Graphics g, int key, int[] p, int y,
			 double val, int bnd, Color co) {
	String s0;
	int len, xx, ww;
	double ga = W/(BAND*400);
	String str = get_str_name(key, p);

	g.setColor(co);
	ww = (int)(val/ga);
	if (ww < 0) {
	    xx = W2+ww;
	    ww = -ww;
	} else
	    xx = W2;
	g.fillRect(xx, y+4, ww, bnd);

	g.setColor(Color.white);
	g.drawString(str, 4, y+FNT);

	s0 = dform(val);
	len = fm.stringWidth(s0);
	g.drawString(s0, W-len-2, y+FNT);
    }
    public String get_str_name(int key, int[] pv) {
	return getKeyName(key) + "_" + getKeyName(key + pv[2]);
    }
    public void disp_lay_line(Graphics g, int max, double dy) {
	int y0;

	g.setColor(Color.gray);
	for (int i = 0; i < max; i += 1) {
	    y0 = (int)(i*dy);

	    g.drawLine(0, y0, W, y0);
	}

	g.drawLine(W2, 0, W2, H);
    }
    //
    public void disp_inha(Graphics g) {
	disp_log_line(g);
	disp_aline(g);

	g.setColor(Color.white);
        for (int i = 0; i < KB; i++) {
	    set_mouse_color(g, i);
            g.fillOval((int)(i*DK), (int)get_inh(i), DOT, DOT);
	}
    }
    public double get_inh(int k) {
	return H2-log_base(Inha[k])*BYH-DSF;
    }
    public double log_base(double value) {
        return Math.log(value);
    }
    public double calc_h(double ih) {
        return H2-log_base(ih)*BYH;
    }
    public void disp_log_line(Graphics g) {
        double h0 = 0.001;
        int hm;

        for (int i = 0; i < 6; i++) {
	    hm = (int)calc_h(h0);

            g.setColor(Color.blue);
            g.drawLine(0, hm, W, hm);

            g.setColor(Color.orange);
	    g.drawString(dform(h0), 2, hm+FNT);

	    h0 *= 10;
        }
    }
    public void disp_aline(Graphics g) {
        int wi;

        for (int i = 0; i < OCTAVE_NAME.length; i++) {
            wi = (int)(DK*i*OCT+DSF);

            g.setColor(Color.blue);
            g.drawLine(wi, 0, wi, H);

            g.setColor(Color.red);
            g.drawString(OCTAVE_NAME[i], wi, H-FNT);
        }
    }
    public void set_mouse_color(Graphics g, int key) {
        if (key == KEY)
            g.setColor(Color.green);
        else if (key < WOUND)
            g.setColor(Color.pink);
        else
            g.setColor(Color.white);
    }
    public String get_octave_name(int n) {
        return OCTAVE_NAME[n/OCT];
    }
    //
    public void disp_cent(Graphics g) {
	disp_aline(g);
	disp_cent_line(g);
	disp_center_line(g);

	double val;
	g.setColor(Color.white);
        for (int i = 0; i < KB; i++) {
	    val = get_cen(AL.getCent(i));
	    //val = get_cen(AL.getICent(i, 1));
            if (val > H || val < 0)
                continue;
            set_mouse_color(g, i);
            g.fillOval((int)(DK*i), (int)val, DOT, DOT);
        }
    }
    public double get_cen(double val) {
        return H2-val*DH-DSF;
    }
    public void disp_cent_line(Graphics g) {
	int py, my = 0;
        double yy, hi;
        double dy = CMAX/8.0;

        for (int i = 1; i < 5; i++) {
            hi = dy*i;
            yy = hi*DH;
            py = (int)(H2-yy);
            my = (int)(H2+yy);

            g.setColor(Color.blue);
            g.drawLine(0, py, W, py);
            g.drawLine(0, my, W, my);

            g.setColor(Color.lightGray);
            g.drawString(dform(hi), 1, py);
            g.drawString(dform(-hi), 1, my);
        }
    }
    public void disp_center_line(Graphics g) {
        g.setColor(Color.gray);
        g.drawLine(0, H2, W, H2);

	g.setColor(Color.lightGray);
	g.drawString("0", 2, H2);
    }
    //
    public void disp_beat(Graphics g) {
	disp_aline(g);
	disp_beat_line(g);
	disp_center_line(g);

	int num, len = INTER.length;
	int[] intn = new int[3];
	double be;
	Color mc;

	for (int i = 0; i < len; i++) {
	    num = INTER[i];
	    mc = IN.getColor(num);
	    intn = IN.getIdata(num);

	    g.setColor(mc);
	    disp_beat_string(g, num, i); 

	    for (int j = 0; j < KB-intn[2]; j++) {
		set_beat_color(g, j, mc);
		be = get_be(AL.getiBeat(j, intn));
		if (be < 0 || be > H)
		    continue;

		g.fillOval((int)(j*DK), (int)be, DOT, DOT);
	    }
	}
    }
    public double get_be(double be) {
	return H2-be*DB-DSF;
    }
    public void disp_beat_string(Graphics g, int num, int i) {
	String ints = IN.toString(IN.getIinte(num));
	int w0 = fm.stringWidth(ints);
	g.drawString(ints, W-w0, FNT*(i+1));
    }
    public void disp_beat_line(Graphics g) {
	int hp = 0, hm, max = 10, xs = 2;
        double hi, hv, db = BMAX/(double)max, dh = H/(double)max;

	for (int i = 1; i < 6; i++) {
            hv = i*db;
            hi = i*dh;
            hm = (int)(H2-hi);
	    hp = (int)(H2+hi);

            g.setColor(Color.blue);
            g.drawLine(0, hm, W, hm);
	    g.drawLine(0, hp, W, hp);

            g.setColor(Color.pink);
            g.drawString(dform(hv), xs+2, hm+FNT);
	    g.drawString("-"+dform(hv), xs, hp);
        }
    }
    public void set_beat_color(Graphics g, int key, Color co) {
        if (key == KEY)
            g.setColor(Color.white);
        else
            g.setColor(co);
    }
    //
    public void disp_trace(Graphics g) {
	int wid = OCT+3;
	int hei = OCT+2; 
	double dw = W/wid;
	double dh = H/hei;
	int[] la = AL.select_laying();
	int[] ke = AL.select_keys();

	disp_trace_line(g, dw, dh, la.length);

	int l = (int)(dw/6);
	double be;
	String ff;
	int x1, x2, y, yd, yf, sft = 33-1;
	for (int i = 0; i < la.length; i++) {
	    x1 = (int)(dw*(ke[i]-sft)+dw);
	    x2 = (int)(dw*(ke[i]+la[i]-sft)+dw);
	    y = (int)(i*dh+dh);
	    yd = y+DSF2;
	    yf = y+FNT+2;

	    g.setColor(Color.white); // white
	    g.drawString(""+(i+1), l, yf);

	    be = AL.getiBeat(ke[i], IN.getIdata(la[i]));
	    ff = dform(be);
	    g.drawString(ff, (x1+x2)/2-fm.stringWidth(ff)/2, yf);

	    g.setColor(IN.getIColor(Math.abs(la[i]))); // color
	    g.drawLine(x1, yd, x2, yd);

	    disp_arrow(g, x2, yd, la[i]);

	    ff = IN.getIinter(la[i]);
	    g.drawString(ff, W-fm.stringWidth(ff)-4, yf); 
	}

	disp_trace_key(g, dw, dh, la.length);
    }
    public void disp_trace_key(Graphics g, double dw, double dh, int len) {
	int x;

	g.setColor(Color.white);
	for (int i = 0; i <= len; i++) {
	    x = (int)(i*dw+dw-2);
	    g.drawString(""+(i+33), x, FNT); // 33~

	    g.drawString(getKey(i+32), x, H-4); // 32~
	}
    }
    public void disp_trace_line(Graphics g, double dw, double dh, int len) {
	int bass = (int)(H-dh);
	int top = (int)dh;
	int dx;

	g.setColor(Color.darkGray);
	for (int i = 0; i <= len; i++) {
	    dx = (int)(dw*i+dw);
	    g.drawLine(dx, top, dx, bass); 
	}

	g.setColor(Color.gray);
	g.drawLine(0, top, W, top);
	g.drawLine(0, bass, W, bass);
    }
    public void disp_arrow(Graphics g, int x, int y, int sw) {
	int sf = 5;
	int ar = (sw < 0) ? x+sf : x-sf;
	int up = y-sf;
	int dw = y+sf;
	g.drawLine(x, y, ar, up);
	g.drawLine(x, y, ar, dw);
    }
}
